//Shaders proxy class
/*
The class is constructed takng as input parameters the path to a bytecode pixel bender file
or an embedded shader class. The clash uses proxy override functions to make a dynamic interface
for changing shaders file parameters providing access with calls like:

shader.parameter = value;

instead of:

shader.setParameter("parameter", value);



Code from the book Foundation ActionScript 3.0 Image Effects by Todd Yard
ISBN-13: 978-1-4302-1872-2
*/




package shaders
{
	import flash.utils.Proxy;
	import flash.utils.flash_proxy;
	import flash.display.Shader;
	import flash.events.Event;
	import flash.events.EventDispatcher;
	import flash.events.IEventDispatcher;
	import flash.net.URLLoader;
	import flash.net.URLLoaderDataFormat;
	import flash.net.URLRequest;
	import flash.utils.ByteArray;

	dynamic public class ShaderProxy extends Proxy implements IEventDispatcher
	{
		private var _shader:Shader;
		private var _eventDispatcher:EventDispatcher;	
		public function ShaderProxy(pathOrClass:Object)
		{
			_eventDispatcher = new EventDispatcher();
			var shaderClass:Class = pathOrClass as Class;
			if(shaderClass!=null)
				createShader(ByteArray(new shaderClass()));
			else if((pathOrClass as String)!=null)
				load(pathOrClass as String);
			else
				throw new Error("Invalid object passed to constructor");	
		}
		
		
		private function createShader(data:ByteArray):void
		{
			_shader = new Shader(data);
			dispatchEvent(new Event(Event.COMPLETE));
		}
		
		private function load(path:String):void
		{
			var loader:URLLoader = new URLLoader();
			loader.dataFormat = URLLoaderDataFormat.BINARY;
			loader.addEventListener(Event.COMPLETE, onShaderLoaded);
			loader.load(new URLRequest(path));
		}
		
		private function onShaderLoaded(event:Event):void
		{
			var loader:URLLoader = event.target as URLLoader;
			createShader(loader.data as ByteArray);
		}
		
		override flash_proxy function getProperty(name:*):*
		{
			if(_shader)
				return getParameter(name);
			return null;	
		}
		
		override flash_proxy function setProperty(name:*,value:*):void
		{
			if(_shader)
				setParameter(name,value);
		}
		
			
		public function getParameter(name:String):Object
		{
			if (_shader.data.hasOwnProperty(name))
			{
				var value:Object = _shader.data[name].value;
				var type:String = _shader.data[name].type;
				if(type == "float" || type=="int")
					value = (value as Array)[0];
				return value;	
			}
			return null;
		}
		
		public function setParameter(name:String, value:Object):Boolean
		{
			if(_shader.data.hasOwnProperty(name))
			{
				if(!(value is Array))
					value = [value];
				_shader.data[name].value = value;
				return true;	
			}
			return false;
		}
		
		public function addEventListener(type:String, listener:Function, useCapture:Boolean=false, priority:int=0, useWeakReference:Boolean=true):void
		{
			_eventDispatcher.addEventListener(type, listener, useCapture, priority, useWeakReference);
		}
		
		public function removeEventListener(type:String, listener:Function, useCapture:Boolean=false):void
		{
			return _eventDispatcher.removeEventListener(type, listener, useCapture);
		}
		
		public function dispatchEvent(event:Event):Boolean 
		{
			return _eventDispatcher.dispatchEvent(event);
		}
		
		public function willTrigger(type:String):Boolean 
		{
			return _eventDispatcher.willTrigger(type);	
		}
		
		public function hasEventListener(type:String):Boolean
		{
			return _eventDispatcher.hasEventListener(type);
		}
		
		public function getShader():Shader
		{
			return _shader;
		}
		
	}
}